from pprint import pprint
import sys
import os
import PySide6
from PySide6.QtWidgets import QApplication, QMainWindow, QDialog, QListWidgetItem, QMessageBox
from PySide6 import QtCore
from PySide6 import QtCharts
from PySide6 import QtWidgets
from charts.create_bar_chart import create_bar_chart
from data.compute_statistics import compute_statistics
from data.create_session import create_session
from data.delete_population_info import delete_population_info
from data.fetch_population_info import fetch_population_info
from data.fetch_region_info import fetch_region_info
from data.fetch_years import fetch_years
from data.insert_population_info import insert_population_info
from data.update_population_info import update_population_info
from dialogs.edit_dialog_code import EditDialog
from mainwindow import Ui_MainWindow

from sqlalchemy import create_engine, text
from sqlalchemy.orm import Session
from statistics import *

from models.population_model import PopulationModel
from dialogs.update_dialog import UpdateDialog

dirname = os.path.dirname(PySide6.__file__)
plugin_path = os.path.join(dirname, 'plugins', 'platforms')
os.environ['QT_QPA_PLATFORM_PLUGIN_PATH'] = plugin_path


class MainWindow(QMainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.model = PopulationModel()
        self.ui.tblView.setModel(self.model)
        self.ui.tblView.horizontalHeader().setSectionResizeMode(
            QtWidgets.QHeaderView.ResizeMode.ResizeToContents
        )

        self.load_regions()
        self.load_years()
        self.load_population()

        self.ui.cmbRegions.currentIndexChanged.connect(self.load_population)
        self.ui.cmbYear.currentIndexChanged.connect(self.load_population)
        self.ui.btnAdd.clicked.connect(self.on_btnAdd_click)
        self.ui.btnRemove.clicked.connect(self.on_btnRemove_click)
        self.ui.btnEdit.clicked.connect(self.on_btnEdit_click)

    def on_btnEdit_click(self):
        item = self.ui.tblView.currentIndex()
        init_data = item.data(QtCore.Qt.ItemDataRole.UserRole)

        dialog = UpdateDialog(self.regions, init_data)
        r = dialog.exec()
        if r == 0:
            return
        
        data = dialog.get_data()

        with create_session()  as s:
            update_population_info(s, data, init_data)

        self.load_population()
        self.load_years()

    def on_btnRemove_click(self):
        item = self.ui.tblView.currentIndex()
        data = item.data(QtCore.Qt.ItemDataRole.UserRole)

        r = QMessageBox.question(self, "Подтверждение", "Точно ли хотите удалить запись?")
        if r == QMessageBox.StandardButton.No:
            return

        with create_session() as s:
            delete_population_info(s, data)

        self.load_population()
        self.load_years()
    
    def on_btnAdd_click(self):
        dialog = EditDialog(self.regions)
        r = dialog.exec()
        if r == 0:
            return

        data = dialog.get_data()
        with create_session() as s:
            insert_population_info(s, data)

        self.load_population()
        self.load_years()

    def load_population(self):
        regions_data = self.ui.cmbRegions.currentData()
        if regions_data:
            region_id = self.ui.cmbRegions.currentData().id
        else:
            region_id = 0

        year = self.ui.cmbYear.currentText()

        self.rows = []

        with create_session() as s:
            rows = fetch_population_info(s, region_id, year)

            for r in rows:
                self.rows.append(r)


        self.model.beginResetModel()

        self.model.setRows(self.rows)
        self.model.setRegions(self.regions)

        self.model.endResetModel()

        self.show_statistics()
        self.draw_line_chart()
        self.draw_pie_chart()
        self.draw_bar_chart()

    
    def draw_bar_chart(self):
        chart = create_bar_chart(self.rows, self.regions)

        self.ui.chartView3.setChart(chart)

    def draw_pie_chart(self):
        self.data_by_regions = {}

        for row in self.rows:
            items = self.data_by_regions.setdefault(row.region_id, [])
            items.append(row)

        series = QtCharts.QPieSeries()
        for region_id, region_data in self.data_by_regions.items():
            region_name = self.regions[region_id].title
            avg = int(mean([i.population for i in region_data]))
            series.append(f"{region_name}", avg)
        
        series.setLabelsVisible()

        chart = QtCharts.QChart()
        chart.addSeries(series)

        self.ui.chartView2.setChart(chart)


    def draw_line_chart(self):
        self.data_by_regions = {}

        for row in self.rows:
            items = self.data_by_regions.setdefault(row.region_id, [])
            items.append(row)

        chart = QtCharts.QChart()
        for region_id, region_data in self.data_by_regions.items():
            region_name = self.regions[region_id].title

            series = QtCharts.QLineSeries()
            series.setName(region_name)
            series.setPointsVisible(True)
            series.setMarkerSize(4)
            for row in region_data:
                series.append(row.year, row.population)
            
            chart.addSeries(series)
            
        chart.createDefaultAxes()
        chart.axisX().setTitleText("Год")
        chart.axisX().setLabelFormat("%i")
        chart.axisX().setMax(chart.axisX().max() + 1)
        chart.axisX().setMin(chart.axisX().min() - 1)
        
        chart.axisY().setLabelFormat("%i")
        chart.axisY().setTitleText("Население, чел.")
        chart.axisY().setMax(chart.axisY().max() + 100000)
        chart.axisY().setMin(chart.axisY().min() - 100000)

        self.ui.chartView1.setChart(chart)

    def show_statistics(self):
        text = compute_statistics(self.rows, self.regions)
        self.ui.lblStatistics.setText(text)


    def load_regions(self):
        self.regions = {}

        with create_session() as s:
            rows = fetch_region_info(s)
            for r in rows:
                self.regions[r.id] = r

        self.ui.cmbRegions.addItem("-")
        for r in self.regions.values():
            self.ui.cmbRegions.addItem(r.title, r)
            
    def load_years(self):
        self.ui.cmbYear.clear()
        self.ui.cmbYear.addItem("-")

        with create_session() as s:
            rows = fetch_years(s)

            for r in rows:
                self.ui.cmbYear.addItem(str(r.year))


if __name__ == "__main__":
    app = QApplication(sys.argv)

    window = MainWindow()
    window.show()

    sys.exit(app.exec())